
/* Chrysalide - Outil d'analyse de fichiers binaires
 * format.c - équivalent Python du fichier "plugins/dex/format.c"
 *
 * Copyright (C) 2012-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "format.h"


#include <assert.h>
#include <pygobject.h>


#include <i18n.h>
#include <plugins/pychrysalide/helpers.h>
#include <plugins/pychrysalide/analysis/binary.h>
#include <plugins/pychrysalide/analysis/content.h>
#include <plugins/pychrysalide/format/executable.h>


#include "constants.h"
#include "../class.h"
#include "../format.h"
#include "../pool.h"



/* Crée un nouvel objet Python de type 'DexFormat'. */
static PyObject *py_dex_format_new(PyTypeObject *, PyObject *, PyObject *);

/* Fournit la liste de toutes les chaînes de la table globale. */
static PyObject *py_dex_format_get_pool_strings(PyObject *, void *);

/* Fournit la liste de tous les types de la table globale. */
static PyObject *py_dex_format_get_pool_types(PyObject *, void *);

/* Fournit la liste de tous les prototypes de la table globale. */
static PyObject *py_dex_format_get_pool_prototypes(PyObject *, void *);

/* Fournit la liste de tous les champs de la table globale. */
static PyObject *py_dex_format_get_pool_fields(PyObject *, void *);

/* Fournit la liste de toutes les méthodes de la table globale. */
static PyObject *py_dex_format_get_pool_methods(PyObject *, void *);

/* Fournit la liste de toutes les classes du format. */
static PyObject *py_dex_format_get_classes(PyObject *, void *);



/******************************************************************************
*                                                                             *
*  Paramètres  : type = type de l'objet à instancier.                         *
*                args = arguments fournis à l'appel.                          *
*                kwds = arguments de type key=val fournis.                    *
*                                                                             *
*  Description : Crée un nouvel objet Python de type 'DexFormat'.             *
*                                                                             *
*  Retour      : Instance Python mise en place.                               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_dex_format_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    PyObject *result;                       /* Instance à retourner        */
    PyObject *content_obj;                  /* Objet pour le contenu       */
    int ret;                                /* Bilan de lecture des args.  */
    GBinContent *content;                   /* Instance GLib du contenu    */
    GExeFormat *format;                     /* Création GLib à transmettre */

    ret = PyArg_ParseTuple(args, "O!", get_python_binary_content_type(), &content_obj);
    if (!ret) return NULL;

    content = G_BIN_CONTENT(pygobject_get(content_obj));

    format = g_dex_format_new(content);

    if (format == NULL)
    {
        result = Py_None;
        Py_INCREF(result);
    }

    else
    {
        result = pygobject_new(G_OBJECT(format));
        g_object_unref(format);
    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit la liste de toutes les chaînes de la table globale.  *
*                                                                             *
*  Retour      : Liste vide ou remplie de chaînes.                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_dex_format_get_pool_strings(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GDexFormat *format;                     /* Version native              */
    uint32_t count;                         /* Nombre d'éléments à traiter */
    uint32_t i;                             /* Boucle de parcours          */
    const char *string;                     /* Chaîne à intégrer           */
    PyObject *str;                          /* Chaîne au format Python     */

    format = G_DEX_FORMAT(pygobject_get(self));

    count = count_strings_in_dex_pool(format);

    result = PyTuple_New(count);

    for (i = 0; i < count; i++)
    {
        string = get_string_from_dex_pool(format, i, NULL);

        if (string == NULL)
        {
            str = Py_None;
            Py_INCREF(str);
        }

        else
            str = PyUnicode_FromString(string);

        PyTuple_SetItem(result, i, str);

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit la liste de tous les types de la table globale.      *
*                                                                             *
*  Retour      : Liste vide ou remplie de types.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_dex_format_get_pool_types(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GDexFormat *format;                     /* Version native              */
    uint32_t count;                         /* Nombre d'éléments à traiter */
    uint32_t i;                             /* Boucle de parcours          */
    GDataType *type;                        /* Type à intégrer             */
    PyObject *tp;                           /* Type au format Python       */

    format = G_DEX_FORMAT(pygobject_get(self));

    count = count_types_in_dex_pool(format);

    result = PyTuple_New(count);

    for (i = 0; i < count; i++)
    {
        type = get_type_from_dex_pool(format, i);

        if (type == NULL)
        {
            tp = Py_None;
            Py_INCREF(tp);
        }

        else
        {
            tp = pygobject_new(G_OBJECT(type));
            g_object_unref(type);
        }

        PyTuple_SetItem(result, i, tp);

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit la liste de tous les champs de la table globale.     *
*                                                                             *
*  Retour      : Liste vide ou remplie de méthodes.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_dex_format_get_pool_fields(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GDexFormat *format;                     /* Version native              */
    uint32_t count;                         /* Nombre d'éléments à traiter */
    uint32_t i;                             /* Boucle de parcours          */
    GBinVariable *variable;                 /* Champ à intégrer            */
    PyObject *var;                          /* Champ au format Python      */

    format = G_DEX_FORMAT(pygobject_get(self));

    count = count_fields_in_dex_pool(format);

    result = PyTuple_New(count);

    for (i = 0; i < count; i++)
    {
        variable = get_field_from_dex_pool(format, i);

        if (variable == NULL)
        {
            var = Py_None;
            Py_INCREF(var);
        }

        else
        {
            var = pygobject_new(G_OBJECT(variable));
            g_object_unref(variable);
        }

        PyTuple_SetItem(result, i, var);

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit la liste de toutes les méthodes de la table globale. *
*                                                                             *
*  Retour      : Liste vide ou remplie de méthodes.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_dex_format_get_pool_methods(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GDexFormat *format;                     /* Version native              */
    uint32_t count;                         /* Nombre d'éléments à traiter */
    uint32_t i;                             /* Boucle de parcours          */
    GDexMethod *method;                     /* Méthode à intégrer          */
    PyObject *meth;                         /* Méthode au format Python    */

    format = G_DEX_FORMAT(pygobject_get(self));

    count = count_methods_in_dex_pool(format);

    result = PyTuple_New(count);

    for (i = 0; i < count; i++)
    {
        method = get_method_from_dex_pool(format, i);

        if (method == NULL)
        {
            meth = Py_None;
            Py_INCREF(meth);
        }

        else
        {
            meth = pygobject_new(G_OBJECT(method));
            g_object_unref(method);
        }

        PyTuple_SetItem(result, i, meth);

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit la liste de tous les prototypes de la table globale. *
*                                                                             *
*  Retour      : Liste vide ou remplie de méthodes.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_dex_format_get_pool_prototypes(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GDexFormat *format;                     /* Version native              */
    uint32_t count;                         /* Nombre d'éléments à traiter */
    uint32_t i;                             /* Boucle de parcours          */
    GBinRoutine *routine;                   /* Routine à intégrer          */
    PyObject *rtn;                          /* Routine au format Python    */

    format = G_DEX_FORMAT(pygobject_get(self));

    count = count_prototypes_in_dex_pool(format);

    result = PyTuple_New(count);

    for (i = 0; i < count; i++)
    {
        routine = get_prototype_from_dex_pool(format, i);

        if (routine == NULL)
        {
            rtn = Py_None;
            Py_INCREF(rtn);
        }

        else
        {
            rtn = pygobject_new(G_OBJECT(routine));
            g_object_unref(routine);
        }

        PyTuple_SetItem(result, i, rtn);

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : self    = objet Python concerné par l'appel.                 *
*                closure = non utilisé ici.                                   *
*                                                                             *
*  Description : Fournit la liste de toutes les classes du format.            *
*                                                                             *
*  Retour      : Liste vide ou remplie de classes.                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_dex_format_get_classes(PyObject *self, void *closure)
{
    PyObject *result;                       /* Valeur à retourner          */
    GDexFormat *format;                     /* Version native              */
    size_t count;                           /* Nombre d'éléments à traiter */
    size_t i;                               /* Boucle de parcours          */
    GDexClass *class;                       /* Classe du format à intégrer */

    format = G_DEX_FORMAT(pygobject_get(self));

    count = g_dex_format_count_classes(format);

    result = PyTuple_New(count);

    for (i = 0; i < count; i++)
    {
        class = g_dex_format_get_class(format, i);
        assert(class != NULL);

        PyTuple_SetItem(result, i, pygobject_new(G_OBJECT(class)));

        g_object_unref(G_OBJECT(class));

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_dex_format_type(void)
{
    static PyMethodDef py_dex_format_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_dex_format_getseters[] = {
        {
            "pool_strings", py_dex_format_get_pool_strings, NULL,
            "Strings inside the Dex pool.", NULL
        },
        {
            "pool_types", py_dex_format_get_pool_types, NULL,
            "Types inside the Dex pool.", NULL
        },
        {
            "pool_prototypes", py_dex_format_get_pool_prototypes, NULL,
            "Prototypes inside the Dex pool.", NULL
        },
        {
            "pool_fields", py_dex_format_get_pool_fields, NULL,
            "Fields inside the Dex pool.", NULL
        },
        {
            "pool_methods", py_dex_format_get_pool_methods, NULL,
            "Methods inside the Dex pool.", NULL
        },
        {
            "classes", py_dex_format_get_classes, NULL,
            "Classes inside the Dex format.", NULL
        },
        { NULL }
    };

    static PyTypeObject py_dex_format_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.format.dex.DexFormat",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT,

        .tp_doc         = "PyChrysalide Dex format",

        .tp_methods     = py_dex_format_methods,
        .tp_getset      = py_dex_format_getseters,
        .tp_new         = py_dex_format_new

    };

    return &py_dex_format_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.format.dex.DexFormat'. *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_dex_format(PyObject *module)
{
    PyTypeObject *py_dex_format_type;       /* Type Python 'DexFormat'     */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_dex_format_type = get_python_dex_format_type();

    dict = PyModule_GetDict(module);

    if (!register_class_for_pygobject(dict, G_TYPE_DEX_FORMAT,
                                  py_dex_format_type, get_python_executable_format_type()))
        return false;

    if (!define_python_dex_format_constants(py_dex_format_type))
        return false;

    return true;

}
